<?php

if (!defined('ABSPATH')) {
    exit;
}

require_once('interface-woocommerce-chatra.php');

/**
 * WooCommerce_Chatra class
 */
class WooCommerce_Chatra implements WooCommerce_Chatra_Interface
{
    const CHATRA_MAX_PRODUCT_LEN = 35;

    // Set up base actions
    public function init()
    {
        add_action('wp_footer', array($this, 'add_code'));
    }

    // Set up public ajax action.
    public function init_public_ajax()
    {
        add_action('wp_ajax_nopriv_wc-chatra-get-cart', array($this, 'ajax_get_cart'));
    }

    // Add JS code after footer
    function add_code()
    {
        // get the current state of the cart
        $params = $this->get_cart();

        // prepare the text for output in JS code
        $params["cart"] = addslashes($params["cart"]);
        $params["cart"] = str_replace("\n", "\\n", $params["cart"]);

        // SOL, Jan 23-2019
        $params["cart"] = str_replace("&nbsp;", "", $params["cart"]);


        // If the user is registered, we get the name and email
        $user = wp_get_current_user();
        if (!empty($user->first_name) || !empty($user->last_name)) {
            $params['name'] = trim($user->first_name . " " . $user->last_name);
            if (empty($params['name'])) {
                unset($params['name']);
            }
        }
        if (!empty($user->user_email)) {
            $params['email'] = $user->user_email;
        }

        $admin_url = admin_url();
        $chatra_code = get_option('chatra-code');

        // Output the script if the code is already connected
        if ($chatra_code) {
            $this->render("make-js-template.php", array(
                "chatra_code" => $chatra_code,
                "admin_url" => $admin_url,
                "params" => $params,
            ));
        }
    }

    // return the cart through the AJAX request
    public function ajax_get_cart()
    {
        $data = $this->get_cart();
        wp_send_json($data);
        wp_die();
    }

    // get the current state of the cart
    private function get_cart()
    {
        $result = array(
            "cart" => "empty",
            "total" => null,
        );

        if (WC()->cart->get_cart_contents_count() > 0) {
            $price_total = wc_prices_include_tax() ? WC()->cart->get_cart_contents_total() + WC()->cart->get_cart_contents_tax() : WC()->cart->get_cart_contents_total();
            $price_total_with_currency = $this->price_with_currency($price_total);

            // SOL
            $price_total_with_currency = str_replace("&nbsp;", "", $price_total_with_currency);

            $items = WC()->cart->get_cart();
            $cart = "";

            foreach ($items as $item => $values) {
                if ("" != $cart) $cart .= "\n";
                $_product = wc_get_product($values['data']->get_id());
                $title = $_product->get_title();
                if (mb_strlen($title) > self::CHATRA_MAX_PRODUCT_LEN) {
                    $title = mb_substr($title, 0, self::CHATRA_MAX_PRODUCT_LEN - 3) . '...';
                }
                $cart .= "{$title}, ";
                $quantity = $values['quantity'];
                if (1 != $quantity) {
                    $cart .= $quantity . "×";
                }
                $price = $this->price_with_currency(get_post_meta($values['product_id'], '_price', true));
                $price = str_replace("&nbsp;", "", $price);
                $cart .= $price;
            }

            $result = array(
                "cart" => $cart,
                "total" => $price_total_with_currency,
            );
        }

        return $result;
    }

    // we return the price with a currency sign, without html tags, as the function get_price () does
    private function price_with_currency($price)
    {
        $price_format = get_woocommerce_price_format();
        $formatted_price = sprintf($price_format, html_entity_decode(get_woocommerce_currency_symbol()), $price);
        return $formatted_price;
    }

    // render template
    public function render($template_name, array $parameters = array(), $render_output = true)
    {
        foreach ($parameters as $name => $value) {
            ${$name} = $value;
        }
        ob_start();
        include __DIR__ . '/../view/' . $template_name;
        $output = ob_get_contents();
        ob_end_clean();

        if ($render_output) {
            echo $output;
        } else {
            return $output;
        }
    }
}
